
import ctypes
import infi.cwrap
import infi.wioctl.api
import infi.wioctl.constants
import infi.wioctl.errors

HANDLE = ctypes.c_void_p
EVT_HANDLE = HANDLE
LPCWSTR = ctypes.c_wchar_p
LPWSTR = ctypes.c_wchar_p
DWORD = ctypes.c_ulong
BOOL = ctypes.c_ulong
POINTER = ctypes.POINTER
PDWORD = POINTER(DWORD)
PVOID = ctypes.c_void_p

TRUE = 1
FALSE = 0

MAX_LENGTH = 256
ERROR_NO_MORE_ITEMS = infi.wioctl.constants.ERROR_NO_MORE_ITEMS

WindowsException = infi.wioctl.errors.WindowsException
InvalidHandle = infi.wioctl.errors.InvalidHandle

def errcheck_invalid_handle():
    def errcheck(result, func, args):
        if result in [None, infi.wioctl.constants.INVALID_HANDLE_VALUE]:
            last_error = ctypes.GetLastError()
            raise InvalidHandle(last_error)
        return result
    return errcheck

class EventLogFunction(infi.cwrap.WrappedFunction):
    return_value = ctypes.c_ulong

    @classmethod
    def get_parameters(cls):
        raise NotImplementedError()

    @classmethod
    def get_errcheck(cls):
        # return infi.cwrap.errcheck_zero()
        return infi.cwrap.errcheck_nonzero()

    @classmethod
    def get_library_name(cls):
        return "Wevtapi.dll"


class EvtOpenLog(EventLogFunction):
    return_value = EVT_HANDLE

    @classmethod
    def get_errcheck(cls):
        return errcheck_invalid_handle()

    @classmethod
    def get_parameters(cls):
        return (
                (EVT_HANDLE, infi.cwrap.IN, "Session"),
                (LPCWSTR, infi.cwrap.IN, "Path"),
                (DWORD, infi.cwrap.IN, "Flags"),
               )

class EvtClose(EventLogFunction):
    return_value = BOOL

    @classmethod
    def get_errcheck(cls):
        return infi.wioctl.api.errcheck_bool()

    @classmethod
    def get_parameters(cls):
        return (
                (EVT_HANDLE, infi.cwrap.IN, "Object"),
               )

class EvtOpenChannelEnum(EventLogFunction):
    return_value = EVT_HANDLE

    @classmethod
    def get_errcheck(cls):
        return errcheck_invalid_handle()

    @classmethod
    def get_parameters(cls):
        return (
                (EVT_HANDLE, infi.cwrap.IN, "Session"),
                (DWORD, infi.cwrap.IN, "Flags"),
               )

class EvtNextChannelPath(EventLogFunction):
    return_value = BOOL

    @classmethod
    def get_errcheck(cls):
        return infi.wioctl.api.errcheck_bool()

    @classmethod
    def get_parameters(cls):
        return (
                (EVT_HANDLE, infi.cwrap.IN, "ChannelEnum"),
                (DWORD, infi.cwrap.IN, "ChannelPathBufferSize"),
                (LPWSTR, infi.cwrap.IN_OUT, "ChannelPathBuffer"),
                (PDWORD, infi.cwrap.IN_OUT, "ChannelPathBufferUsed"),
               )

class EvtQuery(EventLogFunction):
    return_value = EVT_HANDLE

    @classmethod
    def get_errcheck(cls):
        return errcheck_invalid_handle()

    @classmethod
    def get_parameters(cls):
        return (
                (EVT_HANDLE, infi.cwrap.IN, "Session"),
                (LPCWSTR, infi.cwrap.IN, "Path"),
                (LPCWSTR, infi.cwrap.IN, "Query"),
                (DWORD, infi.cwrap.IN, "Flags"),
               )

class EvtNext(EventLogFunction):
    return_value = BOOL

    @classmethod
    def get_errcheck(cls):
        return infi.wioctl.api.errcheck_bool()

    @classmethod
    def get_parameters(cls):
        return (
                (EVT_HANDLE, infi.cwrap.IN, "ResultSet"),
                (DWORD, infi.cwrap.IN, "EventArraySize"),
                (POINTER(EVT_HANDLE), infi.cwrap.IN_OUT, "EventArray"),
                (DWORD, infi.cwrap.IN, "Timeout"),
                (DWORD, infi.cwrap.IN, "Flags"),
                (PDWORD, infi.cwrap.IN_OUT, "Returned"),
               )

# class EvtCreateRenderContext(EventLogFunction):
#     return_value = EVT_HANDLE

#     @classmethod
#     def get_errcheck(cls):
#         return errcheck_invalid_handle()

#     @classmethod
#     def get_parameters(cls):
#         return (
#                 (DWORD, infi.cwrap.IN, "ValuePathsCount"),
#                 (POINTER(LPWSTR), infi.cwrap.IN, "ValuePaths"),
#                 (DWORD, infi.cwrap.IN, "Flags"),
#                )

class EvtRender(EventLogFunction):
    return_value = BOOL

    @classmethod
    def get_errcheck(cls):
        return infi.wioctl.api.errcheck_bool()

    @classmethod
    def get_parameters(cls):
        return (
                (EVT_HANDLE, infi.cwrap.IN, "Context"),
                (EVT_HANDLE, infi.cwrap.IN, "Fragment"),
                (DWORD, infi.cwrap.IN, "Flags"),
                (DWORD, infi.cwrap.IN, "BufferSize"),
                (PVOID, infi.cwrap.IN, "Buffer"),
                (PDWORD, infi.cwrap.IN_OUT, "BufferUsed"),
                (PDWORD, infi.cwrap.IN_OUT, "PropertyCount"),
               )



import os
import ctypes
import infi.cwrap
import textwrap

HANDLE = ctypes.c_void_p
EVT_HANDLE = HANDLE
LPCWSTR = ctypes.c_wchar_p
DWORD = ctypes.c_ulong
BOOL = ctypes.c_ulong
POINTER = ctypes.POINTER
PDWORD = POINTER(DWORD)

TRUE = 1
FALSE = 0

MAX_LENGTH = 256
ERROR_NO_MORE_ITEMS = 259

open_handles = dict()
max_handle_id = 1
available_channel_names = [u"System", u"Application"]

SAMPLE_XML = u"""<Event xmlns='http://schemas.microsoft.com/win/2004/08/events/event'>
<System><Provider Name='Service Control Manager' Guid='{555908d1-a6d7-4695-8e1e-2
6931d2012f4}' EventSourceName='Service Control Manager'/><EventID Qualifiers='16
384'>7036</EventID><Version>0</Version><Level>4</Level><Task>0</Task><Opcode>0</
Opcode><Keywords>0x8080000000000000</Keywords><TimeCreated SystemTime='2010-11-2
1T03:31:33.034185300Z'/><EventRecordID>1</EventRecordID><Correlation/><Execution
 ProcessID='484' ThreadID='564'/><Channel>System</Channel><Computer>37L4247F26-2
4</Computer><Security/></System><EventData><Data Name='param1'>User Profile Serv
ice</Data><Data Name='param2'>stopped</Data><Binary>500072006F006600530076006300
2F0031000000</Binary></EventData></Event>""".replace("\n", "")

class WindowsException(Exception):
    def __init__(self, winerror):
        super(WindowsException, self).__init__()
        self.winerror = winerror

InvalidHandle = WindowsException

class Channel(object):
    def __init__(self, name):
        super(Channel, self).__init__()
        self.name = name

class AvailableChannelsEnum(object):
    def __init__(self):
        super(AvailableChannelsEnum, self).__init__()
        self.items = list(available_channel_names)

class Event(object):
    pass

class QueryEnum(object):
    def __init__(self, name):
        super(QueryEnum, self).__init__()
        self.items = [Event() for i in range(10)]

class RenderContext(object):
    pass

def get_new_handle():
    global max_handle_id
    handle = max_handle_id
    max_handle_id +=1 
    return handle

def EvtOpenLog(session, path, flags):
    assert session is None
    assert path in available_channel_names
    handle = get_new_handle()
    open_handles[handle] = Channel(path)
    return handle

def EvtClose(handle):
    assert handle in open_handles
    open_handles.pop(handle)

def EvtOpenChannelEnum(session, flags):
    assert session is None
    handle = get_new_handle()
    open_handles[handle] = AvailableChannelsEnum()
    return handle

def EvtNextChannelPath(handle, buffer_size, buffer, buffer_used_size):
    assert handle in open_handles
    value = open_handles[handle]
    assert isinstance(value, AvailableChannelsEnum)
    if not value.items:
        raise WindowsException(ERROR_NO_MORE_ITEMS)
    buffer.value = value.items.pop()
    return 1

def EvtQuery(session, path, query, flags):
    assert session is None
    if flags & 1:
        assert path in available_channel_names
    elif flags & 2:
        assert os.path.exists(path)
    handle = get_new_handle()
    open_handles[handle] = QueryEnum(path)
    return handle

def EvtNext(result_set, array_size, array, timeout, flags, returned):
    value = open_handles[result_set]
    assert isinstance(value, QueryEnum)
    assert array_size == 1
    assert timeout == 0
    assert flags == 0
    if not value.items:
        raise WindowsException(ERROR_NO_MORE_ITEMS)
    handle = get_new_handle()
    event = value.items.pop()
    open_handles[handle] = event
    array._obj.value = handle
    return 1

# def EvtCreateRenderContext(values_path_count, values_paths, flags):
#     handle = get_new_handle()
#     open_handles[handle] = RenderContext()
#     return handle

def EvtRender(context, fragment, flags, buffer_size, buffer, buffer_used_size, property_count):
    assert context in open_handles.keys() + [None]
    assert fragment.value in open_handles
    buffer._obj.value = unicode(SAMPLE_XML)
    return 1




from infi.pyutils.contexts import contextmanager
from infi.exceptools import InfiException
from logging import getLogger
from xmltodict import parse

logger = getLogger(__name__)

EvtQueryChannelPath           = 0x1
EvtQueryFilePath              = 0x2
EvtQueryForwardDirection      = 0x100
EvtQueryReverseDirection      = 0x200
EvtQueryTolerateQueryErrors   = 0x1000 
EvtOpenChannelPath   = 0x1
EvtOpenFilePath      = 0x2 
EvtRenderContextValues   = 0
EvtRenderContextSystem   = 1
EvtRenderContextUser     = 2
EvtRenderEventValues   = 0
EvtRenderEventXml      = 1
EvtRenderBookmark      = 2

INFINITE = 2147483647
MAX_BUFFER_SIZE = 2**16

def get_c_api_module():
    from brownie.importing import import_string
    from os import name
    from mock import Mock
    is_windows = name == "nt"
    return import_string("infi.eventlog.c_api" if is_windows else "infi.eventlog.c_api.mock")

c_api = get_c_api_module()

class EventLogException(InfiException):
    pass

class Session(object):
    @contextmanager
    def open_context(self):
        raise NotImplementedError()

class LocalSession(Session):
    @contextmanager
    def open_context(self):
        yield None
        
class RemoteSession(Session):
    def __init__(self, computername, username, password, domain):
        raise NotImplementedError()

class EventLog(object):    
    def __init__(self, session):
        super(EventLog, self).__init__()
        self._session =  session
        self._flags = 0

    @contextmanager
    def open_channel_context(self, channel_name):
        channel_name = unicode(channel_name)
        channels = list(self.get_available_channels())
        flags = 0
        flags |= EvtOpenChannelPath if channel_name in channels else EvtOpenFilePath
        
        with self._session.open_context() as session_handle:
            evt_handle = c_api.EvtOpenLog(session_handle, channel_name, flags)
        try:
            yield evt_handle
        finally:
            c_api.EvtClose(evt_handle)

    def get_available_channels(self):
        with self._session.open_context() as session_handle:
            evt_handle = c_api.EvtOpenChannelEnum(session_handle, 0)
            while True:
                try:
                    buffer = c_api.ctypes.create_unicode_buffer(c_api.MAX_LENGTH)
                    buffer_used = c_api.DWORD()
                    c_api.EvtNextChannelPath(evt_handle, c_api.MAX_LENGTH, buffer, 
                                             c_api.ctypes.byref(buffer_used))
                    yield buffer.value
                except c_api.WindowsException, error:
                    if error.winerror != c_api.ERROR_NO_MORE_ITEMS:
                        raise
                    break

    @contextmanager
    def query_context(self, channel_name, query, flags):
        channel_name = unicode(channel_name)
        with self._session.open_context() as session_handle:
            evt_handle = c_api.EvtQuery(session_handle, channel_name, query, flags)
            try:
                yield evt_handle
            finally:
                c_api.EvtClose(evt_handle)

    @contextmanager
    def next_event_handle_context(self, query_handle):
        event_handle = c_api.EVT_HANDLE()
        returned = c_api.DWORD()
        try:
            c_api.EvtNext(query_handle, 1,
                          c_api.ctypes.byref(event_handle),
                          0, 0, c_api.ctypes.byref(returned))
        except c_api.WindowsException, error:
            if error.winerror != c_api.ERROR_NO_MORE_ITEMS:
                raise
            yield
            return
        try:
            yield event_handle
        finally:
            c_api.EvtClose(event_handle.value)

    # @contextmanager
    # def event_render_context(self, flags):
    #     render_handle = c_api.EvtCreateRenderContext(0, None, flags)
    #     try:
    #         yield render_handle
    #     finally:
    #         c_api.EvtClose(render_handle)

    def render_event(self, event_handle):
    	flags = EvtRenderEventXml
        buffer = c_api.ctypes.create_unicode_buffer(MAX_BUFFER_SIZE)
        buffer_size = MAX_BUFFER_SIZE*c_api.ctypes.sizeof(c_api.ctypes.c_wchar)
        buffer_used = c_api.DWORD()
        property_count = c_api.DWORD()
        c_api.EvtRender(None, event_handle, flags,
                  buffer_size, c_api.ctypes.byref(buffer),
                  c_api.ctypes.byref(buffer_used),
                  c_api.ctypes.byref(property_count))
        return buffer.value
                  
    def event_query(self, channel_name, query="*", reversed=False):
        """:returns: a generator for events, from oldest to newest.
        Use reserved=True to get events in reversed order (newest to oldest)
        """
        channel_name = unicode(channel_name)
        channels = list(self.get_available_channels())
        flags = 0
        flags |= EvtQueryChannelPath if channel_name in channels else EvtQueryFilePath
        flags |= EvtQueryReverseDirection if reversed else EvtQueryForwardDirection
        with self.query_context(channel_name, query, flags) as query_handle:
            while True:
                with self.next_event_handle_context(query_handle) as event_handle:
                    if event_handle is None:
                        break
                    event_data = self.render_event(event_handle).encode("utf-8")
                    is_ascii = lambda c: ord(char) in range(32, 128)
                    ascii_data = ''.join([char if is_ascii(char) else hex(ord(char)) for char in event_data])
                    yield parse(ascii_data)

class LocalEventLog(EventLog):
    def __init__(self):
        super(LocalEventLog, self).__init__(LocalSession())



__version__ = "0.1.4"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.1.4)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = 'abdc897bec6ca411c606fae3f31a9d1117909a6d'
__git_head_subject__ = 'Finished release v0.1.4.'
__git_head_message__ = '* release/v0.1.4:\n  buildout.cfg: adding xmltodict==0.7.0 to requirements\n  buildout.cfg: remove xmltodict from requirements'
__git_dirty_diff__ = ''



SETUP_INFO = dict(
    name = 'infi.eventlog',
    version = '0.1.4',
    author = 'Guy Rozendorn',
    author_email = 'guy@rzn.co.il',

    url = 'http://www.infinidat.com',
    license = 'PSF',
    description = """Bindings to Windows Event Log""",
    long_description = """Bindings to Windows Event Log""",

    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Python Software Foundation License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

    install_requires = ['xmltodict==0.7.0', 'distribute', 'Brownie', 'infi.exceptools', 'infi.cwrap', 'infi.pyutils', 'infi.wioctl', 'mock'],
    namespace_packages = ['infi'],

    package_dir = {'': 'src'},
    package_data = {'': []},
    include_package_data = True,
    zip_safe = False,

    entry_points = dict(
        console_scripts = [],
        gui_scripts = []),
    )

def setup():
    from setuptools import setup as _setup
    from setuptools import find_packages
    SETUP_INFO['packages'] = find_packages('src')
    _setup(**SETUP_INFO)

if __name__ == '__main__':
    setup()




from unittest import SkipTest
from infi import unittest
from infi.eventlog import LocalEventLog

class FacadeTestCase(unittest.TestCase):
    def test__open_system_channel(self):
        eventlog = LocalEventLog()
        with eventlog.open_channel_context("System"):
            pass

    def test__get_available_channels(self):
        eventlog = LocalEventLog()
        channels = list(eventlog.get_available_channels())
        self.assertIn("System", channels)

    def test_get_query_generator(self):
        eventlog = LocalEventLog()
        list(eventlog.event_query("System"))